/******************************************************************************
 * CommandDispatcher.java - Interface for dispatching commands through the system
 * $Id: CommandDispatcher.java,v 1.45 2010/01/23 06:14:49 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: CommandDispatcher.java,v $
 * Revision 1.45  2010/01/23 06:14:49  dick
 * Add YOU_INVITED.
 *
 * Revision 1.44  2009/03/04 19:06:53  dick
 * If you doubleclick on a player in the playerList, then generate a player report request.
 *
 * Revision 1.43  2009/02/24 05:49:56  dick
 * Handle state changes to the ROP flags.
 *
 * Revision 1.42  2009/02/14 12:19:37  dick
 * Add SHOW_ABOUT_DIALOG
 *
 * Revision 1.41  2009/02/12 06:25:25  dick
 * Add BEAR_OFF and PLAY_CUE.
 *
 * Revision 1.40  2009/02/09 22:44:42  dick
 * Add SEND_RESIGN.
 *
 * Revision 1.39  2009/02/06 07:56:48  dick
 * Working on doubles.
 *
 * Revision 1.38  2009/02/05 06:00:00  dick
 * Add TOGGLE_DOUBLE and ROLL_OR_DOUBLE.
 *
 * Revision 1.37  2009/02/03 09:45:04  dick
 * Now that ClientReceiveParser can talk directly to MainDialog, we need our message routes posted here.
 *
 * Revision 1.36  2009/02/01 21:23:23  dick
 * Add SAVED_MATCH and MISS_MANNERS to track other players' behavior.
 *
 * Revision 1.35  2009/01/31 06:11:00  dick
 * Anyone who dispatches commands must also be a ToolbarHandler.
 *
 * Revision 1.34  2009/01/27 19:14:43  dick
 * Add INVITE_WARNING
 *
 * Revision 1.33  2009/01/27 06:57:25  dick
 * Handle the system uninviting us.
 *
 * Revision 1.32  2009/01/22 05:01:48  dick
 * SEND_CHAT_COMMAND becomes the more generic SEND_COMMAND.
 *
 * Revision 1.31  2009/01/18 06:38:25  dick
 * RECEIVE_NETWORK_MESSAGE is obsolete.
 *
 * Revision 1.30  2009/01/18 05:18:06  dick
 * Add dispatch(String, String) and dispatch(Object).
 *
 * Revision 1.29  2009/01/18 04:52:16  dick
 * CommandDispatcher becomes CommandDispatcherImpl.
 * CommandDispatcher is now the interface describing the API for the dispatcher.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS;

import com.buckosoft.fibs.BuckoFIBS.gui.MainDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.board.ToolbarHandler;

/** Interface for dispatching commands through the BuckoFIBS system.
 * @author Dick Balaska
 * @since 2009/01/17
 * @version $Revision: 1.45 $ <br> $Date: 2010/01/23 06:14:49 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/CommandDispatcher.java">cvs CommandDispatcher.java</a>
 */
public interface CommandDispatcher extends ToolbarHandler {
	enum Command {
		SHOW_CONNECTION_DIALOG,
		SHOW_NEW_ACCOUNT_DIALOG,
		DISCONNECT_FROM_NETWORK,
		SHOW_PREFERENCES_DIALOG,
		SHOW_PREFS4MSGS_DIALOG,
		SHOW_ABOUT_DIALOG,
		REGISTER_NEW_USER,			// Includes a roundabout way to CONNECT_TO_SERVER
		BAD_NEW_USER,				// Server says pick someone else
		CONNECT_TO_SERVER,
		SYSTEM_MESSAGE,
		SHUTTING_DOWN,
		NETWORK_CONNECTED,
//		RECEIVE_NETWORK_MESSAGE,
		PLAYER_CHANGED,
		PLAYER_GONE,
		INVITE,
		INVITED,
		INVITE_WARNING,
		WATCHING,
		ACCEPT_INVITATION,
		DECLINE_INVITATION,
		UNINVITED,
		YOU_INVITED,
		START_GAME,
		MATCH_OVER,
		TOGGLE_READY_TO_PLAY,
		READY_TO_PLAY,
		TOGGLE_DOUBLE,
		SEND_MOVE,
		/** Bearing off is automatic, and just does end of turn handling */
		BEAR_OFF,
		SEND_COMMAND,
		YOUR_MOVE,
		ROLL_OR_DOUBLE,
		SEND_ROLL,
		SEND_DOUBLE,
		ACCEPT_OR_DECLINE_DOUBLE,
		SEND_ACCEPT,
		SEND_REJECT,
		SEND_RESIGN,
		/** The 2nd line received from FIBS.  Describe my settings. */
		OWN_INFO,
		/** A Saved Match line from FIBS */
		SAVED_MATCH,
		/** A MissManners warning from FIBS */
		MISS_MANNERS,
		/** Board and double messages get routed through the GameManager */
		GAME_MOVE,
		/** Parse a board received from FIBS */
		FIBS_BOARD,
		/** Either player is pushing the cube */
		DOUBLE,
		WATCH,
		GET_PLAYER_REPORT,
		PLAY_CUE
	}

	void setProperties(BFProperties properties);

	BFProperties	getProperties();
	
	void setMainDialog(MainDialog mainDialog);

	/** Primary entry point to the command dispatcher
	 * @param command The command to execute
	 */
	void dispatch(Command command);

	/** Primary entry point to the command dispatcher
	 * what takes a single string as an argument
	 * @param command The Command to Execute
	 * @param arg1 A string that the dispatched function wants
	 */
	void dispatch(Command command, String arg1);

	/** Primary entry point to the command dispatcher
	 * what takes two strings as an argument
	 * @param command The Command to Execute
	 * @param arg1 A string that the dispatched function wants
	 * @param arg2 Another string that the dispatched function wants
	 */
	void dispatch(Command command, String arg1, String arg2);

	/** Primary entry point to the command dispatcher
	 * what takes an Object as an argument
	 * @param command The Command to Execute
	 * @param obj An Object that the dispatched function wants
	 */
	void dispatch(Command command, Object obj);

	/** Write a message to the System message pane in the specified color.
	 * @param type The style of the text
	 * @param s The message to write
	 */
	public void writeSystemMessage(int type, String s);

	/** Write a message to the System message pane in the normal color.
	 * Terminate the message with a crlf.
	 * @param s The message to write
	 */
	void writeSystemMessageln(String s);

	/** Write a message to the System message pane in the specified color.
	 * @param type The style of the text
	 * @param s The message to write
	 */
	void writeSystemMessageln(int type, String s);

	void writeRegisterUserMessage(String s);

	/** Send a message to the fibs server appending a crlf to the end
	 * @param s The message to send
	 * @return success
	 */
	boolean writeNetworkMessageln(String s);

	/** Write a message to the chat message pane in the normal color.
	 * Terminate the message with a crlf.
	 * @param name The user what sent the message
	 * @param cookie The mode used to send the message (shout, whisper, kibitz, etc)
	 * @param text The message to write
	 */
	void writeChatMessageln(String name, int cookie, String text);
	
	/** Write a message to the game pane appending a crlf to the end.
	 * @param s The message to write
	 */
	void writeGameMessageln(String s);

	/** User has changed the state of the Ready, Online, or Playing button
	 * @param rop The current/new state of the Ready[0], Online[1], and Playing[2] buttons.
	 */ 
	void ropChanged(boolean[] rop);
}
