/******************************************************************************
 * CreateAccountDialog.java - Manage the Create New Account dialog
 * $Id: CreateAccountDialog.java,v 1.8 2009/01/28 18:15:10 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: CreateAccountDialog.java,v $
 * Revision 1.8  2009/01/28 18:15:10  dick
 * Prettier cvs link in the javadoc.
 *
 * Revision 1.7  2009/01/18 05:04:33  dick
 * The owner is a JFrame, not a JDialog.
 *
 * Revision 1.6  2009/01/07 19:55:23  dick
 * Wire up the ProfileSelector.
 *
 * Revision 1.5  2009/01/06 08:07:28  dick
 * Use the server/port panel.
 * Display status as we fill out the form.
 *
 * Revision 1.4  2009/01/06 02:03:25  dick
 * Monitor the filling out of the form and report status.
 *
 * Revision 1.3  2009/01/05 22:32:23  dick
 * Hide the profiles if they are not enabled.
 *
 * Revision 1.2  2009/01/05 07:16:09  dick
 * @see moved to .../account
 *
 * Revision 1.1  2009/01/05 07:09:53  dick
 * ConnectToServerDialog and CreateAccountDialog moved to account.
 *
 * Revision 1.1  2009/01/04 20:00:51  dick
 * Framework for the Create New Account dialog.
 *
 */
package com.buckosoft.fibs.BuckoFIBS.gui.account;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.border.TitledBorder;

import com.buckosoft.fibs.BuckoFIBS.BFProperties;
import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;

/** Manage the Create New Account dialog.
 * @author Dick Balaska
 * @since 2009/01/03
 * @version $Revision: 1.8 $ <br> $Date: 2009/01/28 18:15:10 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/account/CreateAccountDialog.java">cvs CreateAccountDialog.java</a>
 */
public class CreateAccountDialog extends JDialog implements CreateAccountStatusInterface, ProfileSelectorNotifier {
	private static final long serialVersionUID = 1L;
	private	CommandDispatcher	commandDispatcher;
	private	BFProperties		properties;
	Pattern pat = Pattern.compile("[A-Za-z_]+");  //  @jve:decl-index=0:

	private JPanel jPanel = null;
	private JPanel jPanel1 = null;
	private JPanel jPanel2 = null;
	private JLabel jLabel = null;
	private JLabel jLabel1 = null;
	private JTextField jTextFieldUserName = null;
	private JLabel jLabel2 = null;
	private JLabel jLabel3 = null;
	private JPasswordField jPasswordField0 = null;
	private JLabel jLabel4 = null;
	private JPanel jPanelPasswordLabel = null;
	private JLabel jLabel5 = null;
	private JPasswordField jPasswordField1 = null;
	private JButton jButtonCreate = null;
	private JButton jButtonCancel = null;
	private JPanel jButtonPanel = null;
	private JPanel jPanelStatus = null;
	private JTextArea jTextAreaStatus = null;
	private ProfileSelectorPanel jPanelProfile = null;
	private JLabel jLabelProfile = null;
	private JTextPane jTextPaneStatus = null;
	private ServerPortSelectorPanel jPanelServerSelector = null;
	/**
	 * This method initializes 
	 * 
	 */
	public CreateAccountDialog() {
		super();
		initialize();
	}

	public CreateAccountDialog(JFrame owner, CommandDispatcher commandDispatcher) {
		super(owner, true);
		this.commandDispatcher = commandDispatcher;
		this.properties = commandDispatcher.getProperties();
		initialize();
		this.setLocationRelativeTo(owner);
		boolean b = this.properties.isAllowMultiplePersonalities();
		this.jPanelProfile.setVisible(b);
		this.jLabelProfile.setVisible(b);
	}


	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.setSize(new Dimension(407, 294));
        this.setContentPane(getJPanel());
        this.setTitle("Create New Account");
			
	}

	private void createNewAccountButtonActionPerformed(java.awt.event.ActionEvent evt) {
		int profileId = this.jPanelProfile.getSelectedProfile();
//		this.properties.setServerName(profileId,this.jP)
		this.properties.setUserName(profileId, this.jTextFieldUserName.getText());
		this.properties.setPassword(profileId, new String(this.jPasswordField0.getPassword()));
		commandDispatcher.dispatch(CommandDispatcher.Command.REGISTER_NEW_USER, this);
	}

	private void cancelButtonAction() {
		this.setVisible(false);
		dispose();
		commandDispatcher.dispatch(CommandDispatcher.Command.SHOW_CONNECTION_DIALOG);
	}

	private void userNameKeyTyped(java.awt.event.KeyEvent evt) {
		checkActivateCreate();
		
	}
	private void password0KeyTyped(java.awt.event.KeyEvent evt) {
		checkActivateCreate();
		
	}
	private void password1KeyTyped(java.awt.event.KeyEvent evt) {
		checkActivateCreate();		
	}

	private	void caretChanged(javax.swing.event.CaretEvent e) {
		checkActivateCreate();		
		
	}
	private void checkActivateCreate() {
		if (this.jButtonCreate == null)
			return;
		this.jButtonCreate.setEnabled(false);
		String userName = this.jTextFieldUserName.getText();
		if (userName.length() < 3) {
			setStatus("User Name must be at least 3 characters");
			return;
		}
		if (!pat.matcher(userName).matches()) {
			setStatus("Username can have only upper and lower case letters and the underscore");
			return;
		}
		char[] password = this.jPasswordField0.getPassword();
		if (password.length == 0) {
			setStatus("Enter a password");
			return;
		}
		char[] password1 = this.jPasswordField1.getPassword();
		if (password1.length == 0) {
			setStatus("Enter your password again");
			return;
		}
		boolean passwordMismatch = false;
		if (password.length != password1.length)
			passwordMismatch = true;
		else {
			for (int i=0; i<password.length; i++)
				if (password[i] != password1[i]) {
					passwordMismatch = true;
					break;
				}
		}
		if (passwordMismatch) {
			setStatus("You must enter the same password twice");
			return;
		}
		setStatus("");
		this.jButtonCreate.setEnabled(true);
	}
	
	public void setStatus(String s) {
		this.jTextPaneStatus.setText(s);
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.gui.account.CreateAccountStatusInterface#onRegisterFailure()
	 */
	@Override
	public void onRegisterFailure() {
		this.jButtonCreate.setEnabled(false);
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.gui.account.CreateAccountStatusInterface#onRegisterSuccess()
	 */
	@Override
	public void onRegisterSuccess() {
		this.setVisible(false);
		dispose();
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.gui.account.ProfileSelectorNotifier#profileChanged()
	 */
	@Override
	public void profileChanged() {
		int profileId = this.properties.getSelectedProfile();
		this.jTextFieldUserName.setText(this.properties.getUserName(profileId));
		this.jPasswordField0.setText(this.properties.getPassword(profileId));
		this.jPasswordField1.setText(this.properties.getPassword(profileId));
		if (this.jPanelServerSelector != null)
			this.jPanelServerSelector.setServerName(this.properties.getServerName(profileId));
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel() {
		if (jPanel == null) {
			GridBagConstraints gridBagConstraints18 = new GridBagConstraints();
			gridBagConstraints18.fill = GridBagConstraints.BOTH;
			gridBagConstraints18.gridy = 2;
			gridBagConstraints18.weightx = 1.0;
			gridBagConstraints18.weighty = 1.0;
			gridBagConstraints18.gridx = 0;
			GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
			gridBagConstraints31.gridx = 0;
			gridBagConstraints31.fill = GridBagConstraints.BOTH;
			gridBagConstraints31.weighty = 1.0;
			gridBagConstraints31.gridy = 3;
			GridBagConstraints gridBagConstraints21 = new GridBagConstraints();
			gridBagConstraints21.gridx = 0;
			gridBagConstraints21.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints21.weightx = 0.0;
			gridBagConstraints21.insets = new Insets(4, 0, 4, 0);
			gridBagConstraints21.gridy = 3;
			jLabel1 = new JLabel();
			jLabel1.setText("User Name:");
			jLabel = new JLabel();
			jLabel.setText("Create a new account on the selected server. ");
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.weighty = 2.0;
			gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints.weightx = 1.0;
			gridBagConstraints.gridy = 1;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.gridx = 0;
			gridBagConstraints1.fill = GridBagConstraints.BOTH;
			gridBagConstraints1.gridwidth = 1;
			gridBagConstraints1.weightx = 1.0;
			gridBagConstraints1.gridy = 0;
			jPanel = new JPanel();
			jPanel.setLayout(new GridBagLayout());
			jPanel.add(getJPanel1(), gridBagConstraints1);
			jPanel.add(getJPanel2(), gridBagConstraints);
			jPanel.add(getJButtonPanel(), gridBagConstraints21);
			jPanel.add(getJPanelStatus(), gridBagConstraints31);
			jPanel.add(getJTextPaneStatus(), gridBagConstraints18);
		}
		return jPanel;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.gridx = -1;
			gridBagConstraints2.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints2.anchor = GridBagConstraints.CENTER;
			gridBagConstraints2.gridy = -1;
			jPanel1 = new JPanel();
			jPanel1.setLayout(new GridBagLayout());
			jPanel1.add(jLabel, gridBagConstraints2);
		}
		return jPanel1;
	}

	/**
	 * This method initializes jPanel2	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanel2() {
		if (jPanel2 == null) {
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 1;
			gridBagConstraints6.gridy = 1;
			GridBagConstraints gridBagConstraints17 = new GridBagConstraints();
			gridBagConstraints17.gridx = 0;
			gridBagConstraints17.anchor = GridBagConstraints.EAST;
			gridBagConstraints17.gridy = 0;
			jLabelProfile = new JLabel();
			jLabelProfile.setText("Select Profile:");
			GridBagConstraints gridBagConstraints16 = new GridBagConstraints();
			gridBagConstraints16.gridx = 1;
			gridBagConstraints16.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints16.gridy = 0;
			GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
			gridBagConstraints12.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints12.gridy = 5;
			gridBagConstraints12.weightx = 1.0;
			gridBagConstraints12.insets = new Insets(4, 4, 4, 0);
			gridBagConstraints12.gridx = 1;
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.gridy = 1;
			GridBagConstraints gridBagConstraints10 = new GridBagConstraints();
			gridBagConstraints10.gridx = 0;
			gridBagConstraints10.gridy = 5;
			jLabel4 = new JLabel();
			jLabel4.setText("Repeat");
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.fill = GridBagConstraints.BOTH;
			gridBagConstraints8.gridy = 3;
			gridBagConstraints8.weightx = 1.0;
			gridBagConstraints8.insets = new Insets(4, 4, 4, 0);
			gridBagConstraints8.gridx = 1;
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.gridx = 0;
			gridBagConstraints7.anchor = GridBagConstraints.EAST;
			gridBagConstraints7.gridy = 3;
			jLabel3 = new JLabel();
			jLabel3.setText("Password:");
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.gridx = 0;
			gridBagConstraints5.anchor = GridBagConstraints.EAST;
			gridBagConstraints5.gridy = 1;
			jLabel2 = new JLabel();
			jLabel2.setText("Server:");
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.fill = GridBagConstraints.BOTH;
			gridBagConstraints4.gridy = 2;
			gridBagConstraints4.weightx = 1.0;
			gridBagConstraints4.insets = new Insets(4, 4, 4, 0);
			gridBagConstraints4.gridx = -1;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = -1;
			gridBagConstraints3.anchor = GridBagConstraints.EAST;
			gridBagConstraints3.gridy = 2;
			jPanel2 = new JPanel();
			jPanel2.setLayout(new GridBagLayout());
			jPanel2.add(jLabel1, gridBagConstraints3);
			jPanel2.add(getJTextFieldUserName(), gridBagConstraints4);
			jPanel2.add(jLabel2, gridBagConstraints5);
			jPanel2.add(jLabel3, gridBagConstraints7);
			jPanel2.add(getJPasswordField0(), gridBagConstraints8);
			jPanel2.add(getJPanelPasswordLabel(), gridBagConstraints10);
			jPanel2.add(getJPasswordField1(), gridBagConstraints12);
			jPanel2.add(getJPanelProfile(), gridBagConstraints16);
			jPanel2.add(jLabelProfile, gridBagConstraints17);
			jPanel2.add(getJPanelServerSelector(), gridBagConstraints6);
		}
		return jPanel2;
	}

	/**
	 * This method initializes jTextFieldUserName	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getJTextFieldUserName() {
		if (jTextFieldUserName == null) {
			jTextFieldUserName = new JTextField();
			jTextFieldUserName.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyTyped(java.awt.event.KeyEvent e) {
					userNameKeyTyped(e);
				}
			});
			jTextFieldUserName.addCaretListener(new javax.swing.event.CaretListener() {
				public void caretUpdate(javax.swing.event.CaretEvent e) {
					caretChanged(e);
				}
			});
		}
		return jTextFieldUserName;
	}

	/**
	 * This method initializes jPasswordField0	
	 * 	
	 * @return javax.swing.JPasswordField	
	 */
	private JPasswordField getJPasswordField0() {
		if (jPasswordField0 == null) {
			jPasswordField0 = new JPasswordField();
			jPasswordField0.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyTyped(java.awt.event.KeyEvent e) {
					password0KeyTyped(e);
				}
			});
			jPasswordField0.addCaretListener(new javax.swing.event.CaretListener() {
				public void caretUpdate(javax.swing.event.CaretEvent e) {
					caretChanged(e);
				}
			});
		}
		return jPasswordField0;
	}

	/**
	 * This method initializes jPanelPasswordLabel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelPasswordLabel() {
		if (jPanelPasswordLabel == null) {
			jLabel5 = new JLabel();
			jLabel5.setText("Password:");
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.gridx = 0;
			gridBagConstraints9.anchor = GridBagConstraints.EAST;
			gridBagConstraints9.gridy = 0;
			GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
			gridBagConstraints11.gridx = 0;
			gridBagConstraints11.anchor = GridBagConstraints.EAST;
			gridBagConstraints11.gridy = 1;
			jPanelPasswordLabel = new JPanel();
			jPanelPasswordLabel.setLayout(new GridBagLayout());
			jPanelPasswordLabel.add(jLabel4, gridBagConstraints9);
			jPanelPasswordLabel.add(jLabel5, gridBagConstraints11);
		}
		return jPanelPasswordLabel;
	}

	/**
	 * This method initializes jPasswordField1	
	 * 	
	 * @return javax.swing.JPasswordField	
	 */
	private JPasswordField getJPasswordField1() {
		if (jPasswordField1 == null) {
			jPasswordField1 = new JPasswordField();
			jPasswordField1.addKeyListener(new java.awt.event.KeyAdapter() {
				public void keyTyped(java.awt.event.KeyEvent e) {
					password1KeyTyped(e);
				}
			});
			jPasswordField1.addCaretListener(new javax.swing.event.CaretListener() {
				public void caretUpdate(javax.swing.event.CaretEvent e) {
					caretChanged(e);
				}
			});
		}
		return jPasswordField1;
	}

	/**
	 * This method initializes jButtonCreate	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonCreate() {
		if (jButtonCreate == null) {
			jButtonCreate = new JButton();
			jButtonCreate.setText("Create Account");
			jButtonCreate.setEnabled(false);
			jButtonCreate.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					createNewAccountButtonActionPerformed(e);
				}
			});
		}
		return jButtonCreate;
	}

	/**
	 * This method initializes jButtonCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton();
			jButtonCancel.setText("Cancel");
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cancelButtonAction();
				}
			});
		}
		return jButtonCancel;
	}

	/**
	 * This method initializes jButtonPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJButtonPanel() {
		if (jButtonPanel == null) {
			GridBagConstraints gridBagConstraints13 = new GridBagConstraints();
			gridBagConstraints13.gridx = -1;
			gridBagConstraints13.weightx = 0.5;
			gridBagConstraints13.gridy = -1;
			GridBagConstraints gridBagConstraints14 = new GridBagConstraints();
			gridBagConstraints14.insets = new Insets(0, 0, 0, 0);
			gridBagConstraints14.gridy = -1;
			gridBagConstraints14.weightx = 0.5;
			gridBagConstraints14.gridx = -1;
			jButtonPanel = new JPanel();
			jButtonPanel.setLayout(new GridBagLayout());
			jButtonPanel.add(getJButtonCreate(), gridBagConstraints14);
			jButtonPanel.add(getJButtonCancel(), gridBagConstraints13);
		}
		return jButtonPanel;
	}

	/**
	 * This method initializes jPanelStatus	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJPanelStatus() {
		if (jPanelStatus == null) {
			GridBagConstraints gridBagConstraints15 = new GridBagConstraints();
			gridBagConstraints15.fill = GridBagConstraints.BOTH;
			gridBagConstraints15.gridy = -1;
			gridBagConstraints15.weightx = 1.0;
			gridBagConstraints15.weighty = 1.0;
			gridBagConstraints15.gridx = -1;
			jPanelStatus = new JPanel();
			jPanelStatus.setLayout(new GridBagLayout());
			jPanelStatus.setBorder(BorderFactory.createTitledBorder(null, "Status", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Dialog", Font.BOLD, 12), new Color(51, 51, 51)));
			jPanelStatus.setVisible(false);
			jPanelStatus.add(getJTextAreaStatus(), gridBagConstraints15);
		}
		return jPanelStatus;
	}

	/**
	 * This method initializes jTextAreaStatus	
	 * 	
	 * @return javax.swing.JTextArea	
	 */
	private JTextArea getJTextAreaStatus() {
		if (jTextAreaStatus == null) {
			jTextAreaStatus = new JTextArea();
			jTextAreaStatus.setLineWrap(true);
		}
		return jTextAreaStatus;
	}

	/**
	 * This method initializes jPanelProfile	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private ProfileSelectorPanel getJPanelProfile() {
		if (jPanelProfile == null) {
			jPanelProfile = new ProfileSelectorPanel(this, this, properties);
			jPanelProfile.setLayout(new GridBagLayout());
		}
		return jPanelProfile;
	}

	/**
	 * This method initializes jTextPaneStatus	
	 * 	
	 * @return javax.swing.JTextPane	
	 */
	private JTextPane getJTextPaneStatus() {
		if (jTextPaneStatus == null) {
			jTextPaneStatus = new JTextPane();
			jTextPaneStatus.setBackground(SystemColor.controlHighlight);
			jTextPaneStatus.setEditable(false);
		}
		return jTextPaneStatus;
	}

	/**
	 * This method initializes jPanelServerSelector	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private ServerPortSelectorPanel getJPanelServerSelector() {
		if (jPanelServerSelector == null) {
			jPanelServerSelector = new ServerPortSelectorPanel();
			jPanelServerSelector.setLayout(new GridBagLayout());
		}
		return jPanelServerSelector;
	}	

	
}  //  @jve:decl-index=0:visual-constraint="10,10"
