/******************************************************************************
 * ReplayToolbar.java - Be the Replay Toolbar above the board
 * $Id: ReplayToolbar.java,v 1.3 2009/02/14 15:28:49 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2009 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: ReplayToolbar.java,v $
 * Revision 1.3  2009/02/14 15:28:49  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.2  2009/01/31 06:05:25  dick
 * Do something when the buttons are pressed.
 *
 * Revision 1.1  2009/01/28 22:10:33  dick
 * Be the Replay Toolbar above the board.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.board;

import java.awt.Dimension;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JToolBar;

import com.buckosoft.fibs.BuckoFIBS.gui.board.ToolbarHandler.Button;

/** Be the Replay Toolbar above the board
 * @author Dick Balaska
 * @since 2009/01/28
 * @version $Revision: 1.3 $ <br> $Date: 2009/02/14 15:28:49 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/board/ReplayToolbar.java">cvs ReplayToolbar.java</a>
 */
public class ReplayToolbar extends JToolBar {
	private static final long serialVersionUID = 1L;
	private JButton jButtonFirst = null;
	private JButton jButtonBack = null;
	private JButton jButtonNext = null;
	private JButton jButtonEnd = null;

	private	ToolbarHandler	toolbarHandler;  //  @jve:decl-index=0:

	/** Create and initialize the ReplayToolbar
	 */
	public ReplayToolbar() {
		super();
		initialize();
	}
	
	/** Set the object that will receive our button presses.
	 * @param toolbarHandler the toolbarHandler to set
	 */
	public void setToolbarHandler(ToolbarHandler toolbarHandler) {
		this.toolbarHandler = toolbarHandler;
	}


	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.setFloatable(false);
        this.add(getJButtonFirst());
        this.add(getJButtonBack());
        this.add(getJButtonNext());
        this.add(getJButtonEnd());
			
	}

	/**
	 * This method initializes jButtonFirst	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getJButtonFirst() {
		if (jButtonFirst == null) {
			jButtonFirst = new JButton();
			jButtonFirst.setIcon(new ImageIcon(getClass().getResource("/g/begin.png")));
			jButtonFirst.setToolTipText("Show the first move of the game.");
			jButtonFirst.setPreferredSize(new Dimension(18, 18));
			jButtonFirst.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					toolbarHandler.buttonPressed(Button.First);
				}
			});
		}
		return jButtonFirst;
	}

	/**
	 * This method initializes jButtonBack	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getJButtonBack() {
		if (jButtonBack == null) {
			jButtonBack = new JButton();
			jButtonBack.setIcon(new ImageIcon(getClass().getResource("/g/left.png")));
			jButtonBack.setToolTipText("Show the previous move in the game.");
			jButtonBack.setPreferredSize(new Dimension(18, 18));
			jButtonBack.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					toolbarHandler.buttonPressed(Button.Prev);
				}
			});
		}
		return jButtonBack;
	}

	/**
	 * This method initializes jButtonNext	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getJButtonNext() {
		if (jButtonNext == null) {
			jButtonNext = new JButton();
			jButtonNext.setIcon(new ImageIcon(getClass().getResource("/g/right.png")));
			jButtonNext.setToolTipText("Show the next move in the game.");
			jButtonNext.setPreferredSize(new Dimension(18, 18));
			jButtonNext.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					toolbarHandler.buttonPressed(Button.Next);
				}
			});
		}
		return jButtonNext;
	}

	/**
	 * This method initializes jButtonEnd	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getJButtonEnd() {
		if (jButtonEnd == null) {
			jButtonEnd = new JButton();
			jButtonEnd.setIcon(new ImageIcon(getClass().getResource("/g/end.png")));
			jButtonEnd.setToolTipText("Show the last or current move in the game.");
			jButtonEnd.setPreferredSize(new Dimension(18, 18));
			jButtonEnd.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					toolbarHandler.buttonPressed(Button.Last);
				}
			});
		}
		return jButtonEnd;
	}

}
